/*
/*      Calculate WAV file dynamics
/*	   10-26-2014
/*
/*	source: dynamics.c
/*	link to mylib.obj
/*	object: dynamics.exe
/*
/*      Written for MIX PowerC compiler, ver 2.2.0 or later
/*
/*
/*      Compile as medium memory model (use medium.bat)
/*
/*	Revised Aug 2015 to correctly display the percent values.
/*
*/
static char	vnr[14] = "v2.1-08/15";

#include <headers.h>
#include <math.h>

#define  CR   13
#define  BS   8
#define  LTM  62
#define  RTM  60
#define  EOS  '\0'
#define  xesc '\33'

static int	      ESC, i, m, n, ret, term, bits, type;
static int            flag, comp, dflag, flag16, flag24;
static int            value, j, k;
static unsigned long  bin1, bin2, bin3, bin4, bin5, bin6, bin7;
static unsigned long  bin8, bin9, bin10, th0, th1, th2, th3, th4;
static unsigned long  th5, th6, th7, th8, th9, th10, total;
static unsigned int   srate_l, srate_h, header[100];
static unsigned long  count, jj, rate, f_len, savepeak, kk;
static unsigned long  th1, th2, th3, th4, th5, th6, th7, th8, th9, th10;
static char	      t1[80], t2[60], t3[60], temp[80], inbuf[30];
static char           t4[60], t5[60], savetype[40], ref[20];
static char	      putcur[1] = '\0', NL[2] = '\12';
static char	      name1[20], name2[20];
double                pp, peak, ratio, ptime, a, b, c, d, e, dr;
double                g, h, aa, bb, average;
FILE                  *f1;

static char         FF[2] = {'\14','\0'};
static char     CLRSCN[5] = {'\33','[','2','J','\0'};
static char     REVVID[5] = {'\33','[','7','m','\0'};
static char     ATTOFF[5] = {'\33','[','0','m','\0'};

void screen();
void get_hdr();
void process();
void find_data();
void formatl(char *, int);
void printit();
void delay();

main()
	{
m1:	ESC = 0;
	screen();
	if (ESC == 9) goto m9;
        get_hdr();		/* read WAV file header */
        if (ESC == 7) goto m12;
        if (ESC == 9) goto m9;
        process();
        if (ESC == 9) goto m9;
        process2();
        if (ESC == 9) goto m9;
        nextfile();
        if (ESC != 9) {fclose(f1); goto m1;}
m9:     fclose(f1);
        zcrtwr(0x0000, CLRSCN);
	exit(0);
m12:    nextfile();
        if (ESC != 9) {fclose(f1); goto m1;}
        exit(0);
	}

screen(void)
	{
sc0:    flag16 = 0; flag24 = 0;
        bin1 = 0; bin2 = 0; bin3 = 0; bin4 = 0; bin5 = 0;
        bin6 = 0; bin7 = 0; bin8 = 0; bin9 = 0; bin10 = 0;
        zcrtwr(0x0000, CLRSCN);
        zcrtwr(0x0000, CLRSCN);
        zcrtwr(0x021C, "MEASURE WAV FILE DYNAMICS");
        zcrtwr(0x0240, vnr);
        zcrtwr(0x2D19, "FREE NON-COMMERCIAL USE GRANTED");
    zcrtwr(0x2E11, "TDL Technology, Inc., Las Cruces, NM 88012 USA");
        zcrtwr(0x2F1B, "575-405-7289, www.tdl-tech.com");
        formatl("Input file name: ", 8);
        zcrtwr(0x0418, temp);
        zcrtwr(0x0433, ".WAV");
sc1:    zcrtwr2(0x042A, "       ");
        zcrtwr2(0x042A, putcur);
        term = zcrtrd(9, 0x042A, inbuf);
        if (inbuf[0] == xesc) {ESC = 9; return;}
        strcpy(name1, inbuf);
        strcat(name1, ".WAV"); 
        f1 = fopen(name1, "rb");
        if (f1 == 0) {zcrtwr2(0x042A, "        ");
                      strcpy(temp, name1);
                      strcat(temp, " -- file not found");
                      zcrtwr(0x052A, temp); goto sc1;}
        zcrtwr(0x052A, "                              ");
        strcpy(temp, name1);
        strcpy(name2, name1);
        strcat(temp, " -- file open");
        zcrtwr(0x052A, temp);

        formatl("Reference: ", 15);
        zcrtwr(0x071E, temp);
        zcrtwr2(0x072A, "               ");
        zcrtwr2(0x072A, putcur);
        term = zcrtrd(16, 0x072A, inbuf);
        if (inbuf[0] == CR) {strcpy(ref, "               "); goto sc2;}
        if (inbuf[0] == xesc) {ESC = 9; goto sc2;}
        strcpy(ref, inbuf);
sc2:    return;
        }


get_hdr(void)	/* read WAV file header */
	{
        strcpy(t5, "Probably not a wav file");
        bits = 0;
        type = 0;
        n = 0;
hdr0:   j = fgetc(f1);
        if (feof(f1) != 0) {fclose(f1); 
          zcrtwr(0x062A, t5); return;}
        header[n] = j;
        n += 1;
        if (n <= 96) goto hdr0; 
        fclose(f1);		/* close file */


// extract and display header info

        if (header[0] != 'R') {zcrtwr(0x071B, t5); return;}

        flag = 0;
        comp = header[20];	// compression type
        if (comp == 1) {strcpy(t1, "Windows PCM format"); goto hdr1;}
        if (comp == 2) {strcpy(t1, "Microsoft ADPCM"); goto hdr1;}
        if (comp == 6) {strcpy(t1, "a-law format"); flag = 1;
           goto hdr1;}
        if (comp == 7) {strcpy(t1, "mu-law format"); flag = 1;
           goto hdr1;}
        if (comp == 0x11) {strcpy(t1, "IMA ADPCM"); flag = 1;
           goto hdr1;}
        if (comp == 0x16) {strcpy(t1, "ITU G.723 ADPCM"); flag = 1;
           goto hdr1;}
        if (comp == 0x31) {strcpy(t1, "GSM 6.10 format"); flag = 1;
           goto hdr1;}
        if (comp == 0x40) {strcpy(t1, "ITU G.721 ADPCM"); flag = 1;
           goto hdr1;}
        if (comp == 0x50) {strcpy(t1, "MPEG format"); flag = 1;
           goto hdr1;}
        itoa(comp, t2, 10);
        flag = 1;
        strcpy(t1, "Unknown format = ");
        strcat(t1, t2);
hdr1:   strcpy(savetype, t1);
        zcrtwr(0x0919, t1);
        if (flag != 0) {zcrtwr(0x0B19, "No additional information");
           zcrtwr(0x0C19, "available from this program");
           return;}

        j = header[22];
        if (j != 1) {strcpy(temp, "Must be a mono file");
	   zcrtwr(0x0A19, temp); ESC = 9; return;}
        zcrtwr(0x0A19, "MONO"); type = 1; goto hdr2;
        zcrtwr(0x0A19, "Channels unknown");
        ESC = 9;
        return;

hdr2:   j = header[24];
        srate_l = j;
        j = header[25];
        srate_h = j;
        rate = srate_h << 8;
        rate = rate + (srate_l & 0xFF);
        ltoa(rate, t1, 10);
        strcat(t1, " Hz");
        zcrtwr(0x0B19, t1);


        j = header[34];
        if (j == 0x10) {zcrtwr(0x0C19, "16-bit data"); bits = 2;
           flag16 = 1; goto hdr5;}
        if (j == 0x18) {zcrtwr(0x0C19, "24-bit data"); bits = 3;
           flag24 = 1; goto hdr5;}
        zcrtwr(0x0C19, "??-bit data");

hdr5:   strcpy(temp, "Ref: ");
        strcat(temp, ref);
        zcrtwr(0x0D19, temp);

        return;
        }

process(void)     /* read WAV file and calculate dynamic range */
       {
       f1 = fopen(name2, "rb");		/* re-open file */

       n = 100;
pro1:  j = fgetc(f1);
       n -= 1;
       if (n > 0) goto pro1;

       jj = 0;		/* clear positive value counter */
       kk = 0; 		/* clear record counter */
       total = 0;
       savepeak = 1;

	/* HERE FOR 16-BIT DATA */
pro4:  if (flag16 != 1) goto pro8;
pro5:  j = fgetc(f1);		/* read next */
       if (feof(f1) != 0) goto pro9;
pro6:  k = fgetc(f1);		/* read second byte */
       if (feof(f1) != 0) goto pro9;
       kk += 1;
       value = j + (k << 8);
       if (value <= 0) goto pro5;  /* skip negative values */
       jj += 1;			/* count it */
       total += value;		/* running total */
       if (value > savepeak) savepeak = value;
       goto pro5;

pro8:  strcpy(temp, "Not a recognized wav file");
       zcrtwr(0x0F19, temp);
       ESC = 9;
       return;


pro9:  ltoa(savepeak, t1, 10);
       strcpy(temp, "Largest positive peak: ");
       strcat(temp, t1);
       zcrtwr(0x0F19, temp);	/* display peak value */


       average = (double) total / (double) jj;
       ftoa(average, t1, 0, 1, 1);
       strcpy(temp, "Positive average: ");
       strcat(temp, t1);
       zcrtwr(0x1019, temp);	/* average value */


       a = (double) savepeak;
       b = average;
       c = a / b;
       dr = log10 (c) * 20.;
       ftoa(dr, t1, 0, 1, 1);
       strcpy(temp, "Peak / Average: ");
       strcat(temp, t1);
       strcat(temp, " dB");
       zcrtwr(0x1119, temp);	/* display peak/average */

       ltoa(jj, t1, 10);
       ltoa(kk, t2, 10);
       strcpy(temp, "Number of positive values: ");
       strcat(temp, t1);
       zcrtwr(0x1219, temp);
       strcpy(temp, "Number of data words: ");
       strcat(temp, t2);       
       zcrtwr(0x1319, temp);

       a = (double) kk;
       b = (double) rate;
       ptime = a / b;
       strcpy(temp, "Playing time: ");
       ftoa(ptime, t1, 0, 1, 1);
       strcat(temp, t1);
       strcat(temp, " seconds");
       zcrtwr(0x1419, temp);


       fclose(f1);		/* close the file */
       }

       
process2(void)
       {
       jj = 0;       

       f1 = fopen(name2, "rb");   /* re-open file */
 
     /* skip header -- discard 100 bytes */
       n = 100;
pr21:  j = fgetc(f1);
       n -= 1;
       if (n > 0) goto pr21;


    /* HERE FOR 16-BIT DATA */
pr24:  if (flag16 != 1) goto pr28;
       th0 = 0;
       th1 = 0.1 * savepeak;
       th2 = 0.2 * savepeak;
       th3 = 0.3 * savepeak;
       th4 = 0.4 * savepeak;
       th5 = 0.5 * savepeak;
       th6 = 0.6 * savepeak;
       th7 = 0.7 * savepeak;
       th8 = 0.8 * savepeak;
       th9 = 0.9 * savepeak;
       th10 = savepeak;


pr25:  j = fgetc(f1);
       if (feof(f1) != 0) goto pr29;
       k = fgetc(f1);
       if (feof(f1) != 0) goto pr29;
       value = j + (k << 8);
       if (value <= 0) goto pr25;
       jj += 1;
       if (value > th0 && value <= th1) {bin1 += 1; goto pr25;}
       if (value > th1 && value <= th2) {bin2 += 1; goto pr25;}
       if (value > th2 && value <= th3) {bin3 += 1; goto pr25;}
       if (value > th3 && value <= th4) {bin4 += 1; goto pr25;}
       if (value > th4 && value <= th5) {bin5 += 1; goto pr25;}
       if (value > th5 && value <= th6) {bin6 += 1; goto pr25;}
       if (value > th6 && value <= th7) {bin7 += 1; goto pr25;}
       if (value > th7 && value <= th8) {bin8 += 1; goto pr25;}
       if (value > th8 && value <= th9) {bin9 += 1; goto pr25;}
       if (value > th9 && value <= th10) {bin10 += 1; goto pr25;}
       

       strcpy(temp, "No bin was found");
       strcat(temp, "   ");
       ltoa(value, t1, 10);
       strcat(temp, t1);
       zcrtwr(0x0E19, temp);
       

pr28:  strcpy(temp, "Not a recognized wav file");
       zcrtwr(0x0E19, temp);
       ESC = 9;
       return;


pr29:  pp = jj;
       a = (bin1 / pp) * 100.;      
       ftoa(a, t1, 0, 2, 3);
       b = (bin2 / pp) * 100.;
       ftoa(b, t2, 0, 2, 3);
       c = (bin3 / pp) * 100.;
       ftoa(c, t3, 0, 2, 3);
       d = (bin4 / pp) * 100.;
       ftoa(d, t4, 0, 2, 3);
       e = (bin5 / pp) * 100.;
       ftoa(e, t5, 0, 2, 3);
       strcpy(temp, "Percent 0.0 to 0.1 peak: ");
       strcat(temp, t1);
       zcrtwr(0x1719, temp);
       strcpy(temp, "Percent 0.1 to 0.2 peak: ");
       strcat(temp, t2);
       zcrtwr(0x1819, temp);
       strcpy(temp, "Percent 0.2 to 0.3 peak: ");
       strcat(temp, t3);
       zcrtwr(0x1919, temp);
       strcpy(temp, "Percent 0.3 to 0.4 peak: ");
       strcat(temp, t4);
       zcrtwr(0x1A19, temp);
       strcpy(temp, "Percent 0.4 to 0.5 peak: ");
       strcat(temp, t5);
       zcrtwr(0x1B19, temp);

       a = (bin6 / pp) * 100.;
       ftoa(a, t1, 0, 2, 3);
       b = (bin7 / pp) * 100.;
       ftoa(b, t2, 0, 2, 3);
       c = (bin8 / pp) * 100.;
       ftoa(c, t3, 0, 2, 3);
       d = (bin9 / pp) * 100.;
       ftoa(d, t4, 0, 2, 3);
       e = (bin10 / pp) * 100.;
       ftoa(e, t5, 0, 2, 3);
       strcpy(temp, "Percent 0.5 to 0.6 peak: ");
       strcat(temp, t1);
       zcrtwr(0x1C19, temp);
       strcpy(temp, "Percent 0.6 to 0.7 peak: ");
       strcat(temp, t2);
       zcrtwr(0x1D19, temp);
       strcpy(temp, "Percent 0.7 to 0.8 peak: ");
       strcat(temp, t3);
       zcrtwr(0x1E19, temp);
       strcpy(temp, "Percent 0.8 to 0.9 peak: ");      
       strcat(temp, t4);
       zcrtwr(0x1F19, temp);
       strcpy(temp, "Percent 0.9 to 1.0 peak: ");
       strcat(temp, t5);
       zcrtwr(0x2019, temp);


       fclose(f1);
       return;
       }


nextfile(void)
       {
       formatl("P to print, <ESC> to end, N for new file: ", 1);
       zcrtwr(0x2313, temp);
nx1:   zcrtwr2(0x233E, " ");
       zcrtwr2(0x233E, putcur);
       term = zcrtrd(2, 0x233E, inbuf);
       if (inbuf[0] == xesc) {ESC = 9; return;}
       if (inbuf[0] == 'p' || inbuf[0] == 'P') {printit(); goto nx1;}
       if (inbuf[0] == 'n' || inbuf[0] == 'N') {ESC = 0; return;}
       goto nx1;
       }

find_data(void)		// find location of "data" string in header
        {
        dflag = 1;	// found it flag
        n = 0;
fnd1:   if (header[n] == 'd' && header[n+1] == 'a'
           && header[n+2] == 't' && header[n+3] == 'a') goto fnd5;
        n += 1;
        if (n <= 90) goto fnd1;
        dflag = 0;	// didn't find it
fnd5:   return;
        }


formatl(char *bs, int b)	/* format data entry line */
	{
	int j, k;
	j = 0; k = 0;
	while (bs[k] != EOS)
	   {temp[j] = bs[k]; j++; k++;}
	k = 0;
	while (REVVID[k] != EOS)
	   {temp[j] = REVVID[k]; j++; k++;}
	temp[j] = LTM;
	j += 1;
	while (b > 0) {temp[j] = ' '; j++; b--;}
	temp[j] = RTM;
	j += 1;
	k = 0;
	while (ATTOFF[k] != EOS)
	   {temp[j] = ATTOFF[k]; j++; k++;}
	temp[j] = EOS;
	}


printit()
        { 
        strcpy(temp, "PRINTING IN PROGRESS");
        zcrtwr(0x2520, temp);

        strcpy(temp, "      MEASURE WAV FILE DDYNAMICS     ");
        strcat(temp, vnr);
        zprntr(temp);
        zprntr(" ");
        zprntr(" ");
        strcpy(temp, "      File name: ");
        strcat(temp, name1);
        zprntr(temp);
        zprntr(" ");
        strcpy(temp, "      ");
        strcat(temp, savetype);
        zprntr(temp);
        strcpy(temp, "      MONO");
        zprntr(temp);
        if (bits == 2) {strcpy(temp, "      16-bit data"); goto pt1;}
        if (bits == 3) strcpy(temp, "      24-bit data");
pt1:    zprntr(temp);

        strcpy(temp, "      Ref: ");
        strcat(temp, ref);
        zprntr(temp);
        zprntr(" ");

        strcpy(temp, "      Number of positive values: ");
        ltoa(jj, t1, 10);
        strcat(temp, t1);
        zprntr(temp);
        strcpy(temp, "      Number of data words: ");
        ltoa(kk, t1, 10);
        strcat(temp, t1);
        zprntr(temp);
        strcpy(temp, "      Playing time: ");
        a = (double) kk;
        b = (double) rate;
        ptime = a / b;
        ftoa(ptime, t1, 0, 1, 1);
        strcat(temp, t1);
        strcat(temp, " seconds");

        zprntr(temp);
        zprntr(" ");
        zprntr(" ");
        strcpy(temp, "      Largest positive peak: ");
        ltoa(savepeak, t1, 10);
        strcat(temp, t1);
        zprntr(temp);
        strcpy(temp, "      Positive average: ");
        ftoa(average, t1, 0, 1, 1);
        strcat(temp, t1);
        zprntr(temp);
        strcpy(temp, "      Peak / Average: ");
        ftoa(dr, t1, 0, 1, 1);
        strcat(temp, t1);
        strcat(temp, " dB");
        zprntr(temp);
        zprntr(" ");
        zprntr(" ");

        pp = (double) jj;
        a = (double) bin1;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.0 to 0.1 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin2;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.1 to 0.2 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin3;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.2 to 0.3 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin4;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.3 to 0.4 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin5;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.4 to 0.5 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin6;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.5 to 0.6 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin7;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.6 to 0.7 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin8;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.7 to 0.8 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin9;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.8 to 0.9 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        a = (double) bin10;
        b = (a / pp) * 100.;
        strcpy(temp, "      Percent 0.9 to 1.0 peak: ");
        ftoa(b, t1, 0, 2, 3);
        strcat(temp, t1);
        zprntr(temp);

        delay();
        zcrtwr(0x2520, "                        ");

/* some systems need this to print the page, others will print
/* an extra blank page. Uncomment this line if needed.
/*      zprntr(&FF);
*/
        }


delay(void)
        {
        count = 400000000;
dy1:    count -= 1;
        if (count > 0) goto dy1;
        }

